@@ -120,7 +120,7 @@ setMethod("predict", "unmarkedFitList", function(object, type, newdata=NULL,
         ese <- lapply(fitList, predict, type = type, newdata = newdata,
             backTransform = backTransform, level=level)
 
-        if(class(newdata) == "RasterStack"){
+        if(inherits(newdata, "RasterStack")){
           if(!require(raster)) stop("raster package is required")
           ese <- lapply(ese, as.matrix)
         }
@@ -139,7 +139,7 @@ setMethod("predict", "unmarkedFitList", function(object, type, newdata=NULL,
         out$lower <- as.numeric(lower %*% wts)
         out$upper <- as.numeric(upper %*% wts)
 
-        if(class(newdata) == "RasterStack"){
+        if(inherits(newdata, "RasterStack")){
           E.mat <- matrix(out[,1], dim(newdata)[1], dim(newdata)[2], byrow=TRUE)
           E.raster <- raster::raster(E.mat)
           raster::extent(E.raster) <- raster::extent(newdata)
@@ -150,7 +150,7 @@ setMethod("predict", "unmarkedFitList", function(object, type, newdata=NULL,
             raster::extent(i.raster) <- raster::extent(newdata)
             out.rasters[[i]] <- i.raster
           }
-          out.stack <- stack(out.rasters)
+          out.stack <- raster::stack(out.rasters)
           names(out.stack) <- colnames(out)
           raster::crs(out.stack) <- raster::crs(newdata)
           return(out.stack)
